/*
 * Copyright 2018 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.drools.workbench.screens.scenariosimulation.client.commands;

import com.google.gwtmockito.GwtMockitoTestRunner;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyString;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

@RunWith(GwtMockitoTestRunner.class)
public class EnableRightPanelCommandTest extends AbstractCommandTest {



    private EnableRightPanelCommand enableRightPanelCommand;

    private static final String FACT_NAME = "FACT_NAME";

    @Before
    public void setup() {
        super.setup();
    }

    @Test
    public void executeWithoutFactName() {
        enableRightPanelCommand = new EnableRightPanelCommand(rightPanelPresenterMock);
        enableRightPanelCommand.execute();
        verify(rightPanelPresenterMock, times(1)).onEnableEditorTab();
        verify(rightPanelPresenterMock, never()).onEnableEditorTab(anyString(), any(), eq(false));
    }

    @Test
    public void executeWithFactName() {
        enableRightPanelCommand = new EnableRightPanelCommand(rightPanelPresenterMock, FACT_NAME, null, true);
        enableRightPanelCommand.execute();
        verify(rightPanelPresenterMock, times(1)).onEnableEditorTab(eq(FACT_NAME), eq(null), eq(true));
        verify(rightPanelPresenterMock, never()).onEnableEditorTab();
    }
}